import pytest
from schema import Schema

from cazart import bad_json, validation_error


def test_validates(cazart):
    result = None

    @cazart.route("/", schema=Schema({"name": str}))
    def root(res):
        nonlocal result
        result = res
        return ("ok", 200)

    with cazart.flask.test_client() as client:
        resp = client.get("/", json={"name": "foo"})
        assert resp.data == b"ok"
        assert resp.status_code == 200
        assert result == {"name": "foo"}


def test_does_not_validate(cazart):
    @cazart.route("/", schema=Schema({"name": str}))
    def root(res):
        return ("ok", 200)

    with cazart.flask.test_client() as client:
        resp = client.get("/", json={"name": 1234})
        assert resp.data == b"Key 'name' error:\n1234 should be instance of 'str'"
        assert resp.status_code == 400
        assert bad_json == {"name": 1234}
        assert validation_error == "Key 'name' error:\n1234 should be instance of 'str'"


def test_missing_json(cazart):
    @cazart.route("/", schema=Schema({"name": str}))
    def root(res):
        return ("ok", 200)

    with cazart.flask.test_client() as client:
        resp = client.get("/")
        assert resp.data == b"missing or malformed JSON"
        assert resp.status_code == 400


def test_malformed_json(cazart):
    @cazart.route("/", schema=Schema({"name": str}))
    def root(res):
        return ("ok", 200)

    with cazart.flask.test_client() as client:
        resp = client.get("/", data="{'bad json'}")
        assert resp.data == b"missing or malformed JSON"
        assert resp.status_code == 400


@pytest.mark.parametrize(
    ("method", "json", "exp_data", "status_code", "exp_result"),
    [
        ("GET", {"id": 1234}, b"ok", 200, {"id": 1234}),
        ("GET", {"name": "foo"}, b"Missing key: 'id'", 400, None),
        ("POST", {"name": "foo"}, b"ok", 200, {"name": "foo"}),
        ("POST", {"id": 1234}, b"Missing key: 'name'", 400, None),
    ],
)
def test_routes_methods(cazart, method, json, exp_data, status_code, exp_result):
    result = None

    @cazart.route("/", schema={"GET": Schema({"id": int}), "POST": Schema({"name": str})})
    def root(res):
        nonlocal result
        result = res
        return ("ok", 200)

    with cazart.flask.test_client() as client:
        resp = client.open(method=method, json=json)
        assert resp.data == exp_data
        assert resp.status_code == status_code

        if exp_result is not None:
            assert result == exp_result



from functools import wraps

from flask import Flask, request
from schema import SchemaError
from werkzeug import Local, LocalManager

_local = Local()
_local_manager = LocalManager()

bad_json = _local("bad_json")
validation_error = _local("validation_error")


def _fail_on_invalid():
    return (_local.validation_error, 400)


def _update_locals(json, message):
    _local.bad_json = json
    _local.validation_error = message


class Cazart:
    """
    A thin wrapper for Flask and schema, providing schema-validated endpoint decorators.
    """

    def __init__(self, import_name, **kwargs):
        self.import_name = import_name
        self.flask = Flask(import_name, **kwargs)
        _local_manager.make_middleware(self.flask)

    def route(self, rule, *, schema, error=_fail_on_invalid, **kwargs):
        """
        Like ``Flask.route()``, but with an additional schema to validate against.
        Adds two keywords, ``schema`` and ``error``, and passes all other arguments
        to Flask.

        :param schema: The schema to validate against, or a dictionary of HTTP methods to schemas
        :type schema: ``schema.Schema`` or dict
        :param error: The function to call on validation failure
        :type error: function
        """
        if isinstance(schema, dict):
            kwargs["methods"] = schema.keys()

        def wrapper(fn):
            @wraps(fn)
            @self.flask.route(rule, **kwargs)
            def decorated():
                nonlocal schema
                res = request.get_json(force=True, silent=True)
                if res is None:
                    _update_locals(res, "missing or malformed JSON")
                    return error()

                if isinstance(schema, dict):
                    schema = schema.get(request.method)

                try:
                    schema.validate(res)
                except SchemaError as e:
                    _update_locals(res, e.code)
                    return error()
                return fn(res)

            return decorated

        return wrapper



#!/usr/bin/env python3

from setuptools import setup

version = {}
with open("./src/cazart/version.py") as f:
    exec(f.read(), version)

with open("./README.md") as f:
    long_description = f.read()

setup(
    name="cazart",
    version=version["__version__"],
    license="MIT",
    author="William Woodruff",
    author_email="william@yossarian.net",
    description="Flask + Schema = Cazart!",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/woodruffw/cazart",
    packages=["cazart"],
    package_dir={"": "src"},
    platforms="any",
    python_requires=">=3.6",
    install_requires=[
        "flask",
        "schema",
    ],
    extras_require={
        "dev": [
            "flake8",
            "black",
            "isort[pyproject]",
            "pytest",
            "coverage",
            "twine",
        ]
    }
)
